# gawkbug.awk - port of GNU Awk 5.2 gawkbug.in script to MS-Windows environment

# gawkbug - create a bug report and email it to the bug address
# All email defaults to bug-gawk@gnu.org.
#
# Copyright (C) 2022 Free Software Foundation, Inc. and is free software.
# (the GPL v3 copyright text is documented at the end of this file)
#---------------------------------------------------------------------------
#
# John Naman, gawker@703n.com, Public Domain
# September 2022; Dec 2022
# History: started with \gawk-5.2.0\gawkbug.in distribution;
#  plus reference to the similar 'bashbug' script
#---------------------------------------------------------------------------

# output of this gawk script is intended to conform to:
# (Gawk UserGuide) Section B.4.2 Submitting Bug Reports
# and (Gawk UserGuide) B.4.6 Reporting Problems with Non-Unix Ports
# (Gawk UserGuide) is "GAWK: Effective AWK Programming:
#     A Users Guide for GNU Awk Edition 5.2 August, 2022 Arnold D. Robbins"
# See https://www.gnu.org/software/gawk/manual/	# HTML,ASCII, PDF,TeX
#---------------------------------------------------------------------------

# usage() ported from GNU Linux gawkbug.in
function usage()
{
printf("Usage: gawk -f gawkbug.awk [-- outputfile]\n" \
       "Usage: gawk -f gawkbug.awk -- [--help] [--version]\n" \
       "   or: gawk -E gawkbug.awk [--help] [--version] \n" \
       "\n" ) > "/dev/stderr"
printf("Gawkbug is used to report bugs to the Gawk maintainers\n" \
       "   for when Gawk doesn't behave like you'd like, or expect.\n" \
       "All email defaults to bug-gawk@gnu.org.\n" \
       "PLEASE do NOT try to report bugs by posting in comp.lang.awk,\n" \
       "   or by using a web forum such as Stack Overflow.\n" \
       "For full instructions, see `Reporting Problems and Bugs' in the\n" \
       "Users Guide for GNU Awk. This same information may be found at\n" \
       "https://www.gnu.org/software/gawk/manual/html_node/Bugs.html.\n" \
       "\n" ) > "/dev/stderr"
printf("Gawkbug will start up your editor (as defined by the shell's\n" \
       "   EDITOR environment variable) with a preformatted bug report\n" \
       "   template for you to fill in. \n" \
       "In Windows, you must copy the template text into an email program\n" \
       "   after saving your report with your description of the problem.\n" \
       "\n" ) > "/dev/stderr"
	exit _abend=1
}
#---------------------------------------------------------------------------

BEGIN {
	dBug = 1
	split("", rForm, FS)
	split("", pgmFeature, FS)	# init arrays for --lint
	rForm["gawkbug ver:"] = "win.22.1226"	#  THIS gawkbug.awk script version
	#>src: C:\Npgm\Gawkbug\gawkbug.Tr26.awk "0.22.1226"

	pgmFeature["choice"] = 0	# False => do NOT invoke user choice feature +221214

	pgmFeature["eform_new"] = 1	# default, can be overriden
	pgmFeature["validate"] = 1	# default, can be overriden (why? just messages)

	rForm["Date:"] = get_datetimeUTC()	# global gawkbug datestamp for this run
	# ensure gawkbug is executable & avoid fatal errors
	if (gawk__sandbox()) {
		print("This version of gawkbug can NOT run in --sandbox mode!") > "/dev/stderr"
		exit _abend=3
	}
	get_user_options()
	if (pgmFeature["eform_new"]) {
		# use editor as defined by the EDITOR environment variable or default
		Editor = ("EDITOR" in ENVIRON) && (ENVIRON["EDITOR"] != "")
		sysfn["editor"] = ("EDITOR" in ENVIRON) && (ENVIRON["EDITOR"] != "") ? ENVIRON["EDITOR"] : "notepad.exe"
		# "notepad" is the default Win editor
		#>aka: "c:\windows\system32\notepad.exe"
		#>aka: "%SYSTEMROOT%\notepad.exe"  #>fyi:%SYSTEMROOT% ==  %WINDIR%
		#------------------------

		#*********************************************************************
		# *** create  preformatted bug report template for user to fill in ***
		#*********************************************************************
		init_rptLabels()	# create template for rptLabels[]
		# rptLabels[] array has the order autofilled, e.g., rptLabels[2]="To:"
		# Doc: LAYOUT of rForm[]array 'From:|To:|Subject:|Date:||
		# Configuration Information [Automatically generated, do not change]:|
		# |Gawk Version:|Machine:|OS:|platform:|Compiler:|Compilation CFLAGS:|
		# uname output:|LC_ALL:|I have read: |  Yes or No:|gawkbug ver:|
		# Description:|Repeat-By:|Fix:'
		# note: || in rForm ==> blank line in output (desired here)
		#.......................................................
		
		# some eform fields already filled in by call to get_user_options() 
		# Configuration Information [Automatically generated, do not change]:
		# fill-in some config info:
		# [Gawk Version:|Machine:|OS:|platform:|LC_ALL:| etc.]
		get_ConfigInfo()
		make_boilerplate()	# [I have:|Description:|Repeat-By:|Fix:|Subject:]
		# !!! output needs to goto file to be edited then emailed ...

		# OUTPUT IN CORRECT ORDER given by the global array rptLabels[1]="From:" ...
		PROCINFO["sorted_in"] = "@ind_num_asc"	# Gawk specific, but so is gawkbug report
		for (i in rptLabels) {
			printf("%s %s\n", rptLabels[i], rForm[rptLabels[i]]) > sysfn["edit_fnam"]
			# note: it is okay that some labels are null "" for line spacing
		}
		#>ie: rptLabels[2]="To:"; rForm["To:"]="bug-gawk@gnu.org";
		printf("#========\n") > sysfn["edit_fnam"]
		#^^^ line added to show end of eform ^^^
		# close file before user editing
		close_winfile(sysfn["edit_fnam"], "closing eform from BEGIN block")
		#****************************************************************************
		#  invoke editor with a preformatted bug report template for user to fill in.
		#  user should edit gawkbug report in an editor; fill in the blanks; and save
		#****************************************************************************
		str = "echo Now edit your gawkbug report and be sure to SAVE  or quit editor to cancel!"
		system(str " & pause")	# pause to give user time to read
		# *** user edits template here *** 
		system(sysfn["editor"] " " sysfn["edit_fnam"])
		#  user can save a bug report or quit editor without saving any changes
		
		#**********************************************************
		#*** read saved report back in, check for valid entries ***
		#**********************************************************
		#  the easy way is to finish the BEGIN block and then let gawk process
		#  validity checks in main block as with any other input file
		#  add edit filename to ARGV so main block will process the file
		#  normally expect: ARGIND=0; ARGC=1; ARGV[]= [0]='gawk'; code generalized:
		ARGV[ARGIND + 1] = sysfn["edit_fnam"]
		ARGC++
	}
	# ^^^ created and edited new eform ^^^
	init4validation()	# init globals for validation in main loop
	#>Global regex to validate email address on From: [YOUR email address here]
	rx_valid_email = @/[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@([a-z0-9]([a-z0-9-]*[a-z0-9])\.)+[a-z0-9]([a-z0-9-]*[a-z0-9])/	# BEST w/ Valid & Invalid addresses
	# permission to validate?
	if (! pgmFeature["validate"]) {
		exit 0
	}
# handle MS-Windows text files with carriage-return plus line-feed line endings.
	RS = "\r?\n"
# after end of BEGIN block, the main loop will validate the user edited
# eform file named on command line or default: gawkbug.eform.yymmdd.hhmmdd.txt
}
# end BEGIN block
#----------------------

#>|=========================================================================
#>| *** MAIN *** MAIN *** MAIN *** MAIN *** MAIN *** MAIN *** MAIN *** MAIN
#>|=========================================================================
#  Validate user edited eform file (gawkbug.eform.yymmdd.hhmmdd.txt)
/^[[:space:]]*$/ {
	next
}

{
	# attempt a global validity check: has eform line been changed?
	check4change()	# may set global currlabel
}

/^From:/ {
	if (nochange[currlabel]) {
		emsg("FATAL ERROR:", "You have not given your FROM: email address")
	}
	if (currvalue ~ rx_valid_email) {
		next
	}	# probably okay
	# default
	emsg("warning:", "your FROM: email address may not be valid.")
	next
}

# skip: /^gawkbug ver:/	/^Date:/	/^To:/ # User decides email address
# skip: /^Compiler:/	/^Compilation CFLAGS:/	/^uname output:/	/^LC_ALL:/
# skip: /^Configuration Information \[Automatically generated, do not change\]:/
/^Subject:/ {
	if (nochange[currlabel]) {
		emsg("FATAL ERROR:", "You have not changed the Subject: from the default.\nPlease use a more descriptive subject header.")
	}
	#linux:	echo "$0: You have not changed the subject from the default."
	#linux:	echo "$0: Please use a more descriptive subject header."
	next
}

/^(Gawk Version|Machine|OS|platform):/ {
	if (! nochange[currlabel]) {
		emsg("warning:", "do not change Automatically generated Configuration Information")
	}
	next
}

/^  Yes or No:/ {
	if (nochange[currlabel]) {
		emsg("warning:", "You have not answered Yes or No to read Bugs.html")
	}
	next
}

/^Description:/ {	# [Detailed description of the problem, suggestion, or complaint.]
	if (nochange[currlabel]) {
		emsg("FATAL ERROR:", "You have not provided a Description.")
	}
	next
}

{
	# any skips and user text lines presumed valid
	next
}

#>|---------------------------------------------------------------------
#>| *** End MAIN *** End MAIN *** End MAIN *** End MAIN *** End MAIN ***
#>|---------------------------------------------------------------------

#>|================================================================
#>| *** ENDFILE *** ENDFILE *** ENDFILE *** ENDFILE *** ENDFILE ***
#>|================================================================
ENDFILE {	# after eform is validated, maybe allow user to re-edit
	if (! pgmFeature["choice"] || valid[0] == 0) {
		exit 0	# ENDFILE is done; goto END{} block
	}
	if (pgmFeature["choice"] && valid[0] > 0) {
		# choice syntax c.f. https://www.robvanderwoude.com/choice.php
		#  /C are choice(s); /D is default choice;/T is n seconds wait before default
		#  /M is message before wait for user response
		ask = "CHOICE /C YN /T 9 /D Y /M \"Edit your gawkbug report again? (Y or N default=Yes):\""
		rc = system(ask)
		if (rc == 2) {
			exit 0	# ENDFILE is done; goto END{} block
		}
		if (rc == 1) {	# Y=1; N=2
			#  edit again
			system("echo be sure to SAVE your gawkbug report or quit editor to cancel! & pause")
			#*** user re-edits template here ***
			system(sysfn["editor"] " " sysfn["edit_fnam"])
			#  user can save a bug report or quit editor without saving any changes
			#*** setup to read saved report back in, check for valid entries ***
			ARGV[ARGIND + 1] = sysfn["edit_fnam"]
			ARGC++
			init4validation()	# init globals for validation after every edit
			# ENDFILE SHOULD now rewind and restart main processing
		}		# eo rc==1
	}	# end of pgmFeature["choice"] && valid[0] > 0
}
# end ENDFILE block
#---------------------------------------
#
function check4change(i, lbln, lbl)
{
	# attempt at global validity check
	currlabel = ""
	currvalue = ""
	for (i in rForm) {
		lbln = index($0, ":")	#can't use $1 because some lbls have w.s.
		if (lbln < 2) {
			break	# not a label
		}
		lbl = substr($0, 1, lbln)
		if (lbl == i) {	# have match on `label:`
			currvalue = substr($0, lbln + 2)	# account for space
			if (currvalue == rForm[i]) {	# == means have NO change
				nochange[i] = 1	# flag it; some are okay NOT to change, e.g. config
			}
			currlabel = i	# save for other validity checks
			return	# done; change or not, label matched
		}		# have match
	}	# for(i in rForm)
	# no label match on begining of $0, so reset the curr vars
	currlabel = currvalue = ""
}
#-------------------------------------------
#
function chk_eform_filename(efn)
{
	# only checks for valid filename.ext, NOT full UNC path
	# Windows (FAT32, NTFS) filename (after path): Any Unicode except
	# NUL, \, /, :, *, ?, ", <, >, |. " 
	# Also, no space char at the start or end, and no period at the end.
	rxWinFilename = @/([^*/:?|<>\\\x00-\x20"][^*/:?|<>\\\x00-\x1F"]*)?[^*/:?|<>\\.\x00-\x20"]$/	#" 
	# !!! Never "/dev/stdout" or "/dev/stderr" !!! due to main validation reads
	if (efn ~ /^\/dev\/std(out|err)$/) {
		return ""
	}
	if (efn ~ rxWinFilename) {
		return efn
	}
	return ""
}
#-------------------------------------------
#
function close_winfile(fnam, emsg, sysrc)
{
	# utility: close windows file for gawkbug
	sysrc = close(fnam)
	if (sysrc != 0) {
		printf("Error: %s closing file: '%s'; rc=%s; ERRNO=%s;\n", emsg, fnam, sysrc, ERRNO) > "/dev/stderr"
	}	# error on close()
	return sysrc	# if caller wants to know ...
}
#-------------------------------------------
#
function emsg(class, text, i, em)
{
	# validation error & warning messages
	#>Note: emsg(class,text) always does a next statement.
	#>Expect: "FATAL ERROR:" or "warning:"
	errsarr[class]++	# count # emsgs
	valid[0] = i = length(valid)	# for append & count
	valid[i] = em = sprintf("%s %s", class, text)
	print(em) > "/dev/stderr"
	print($0 "\n") > "/dev/stderr"	# print $0 under error msg
	errsarr[class]++	# count # emsgs
	next	# do not return to caller
}
#-------------------------------------------
#
function gawkVersion(retn)
{
	# returns text string
	retn = sprintf("GNU Awk %s,", PROCINFO["version"])
	retn = retn sprintf(" API %s.%s,", PROCINFO["api_major"], PROCINFO["api_minor"])
	retn = retn sprintf(" (%s, %s)", PROCINFO["mpfr_version"], PROCINFO["gmp_version"])
	#  'GNU Awk 5.2.1, API 3.2, (GNU MPFR 4.0.2, GNU MP 6.1.2)'
	# diy: PROCINFO["pma"]=("GAWK_PERSIST_FILE" in ENVIRON)&&(ENVIRON["GAWK_PERSIST_FILE"]!="")
	return retn
}
#-------------------------------------------

# gawk__sandbox --- test if gawk is running in --sandbox mode
# Returns:
#     1 = running in Sandbox mode!;sets global PROCINFO["sandbox"] = 1 also;
#     0 = normal mode, NOT running in Sandbox mode.PROCINFO["sandbox"] = 0 also;
#okay: if(gawk__sandbox()){print "running in Sandbox mode!"}
#               else {system("echo normal mode, not sandbox")}
function gawk__sandbox(i, n, retn)
{
	retn = ""	# required to init by --lint
	n = length(PROCINFO["argv"]) - length(ARGV)
	for (i = 1; i < n; i++) {	# compare using == vs ~ requires IGNORECASE=1
		if (PROCINFO["argv"][i] == "--sandbox") {
			retn = 1
			break
		}
		if (PROCINFO["argv"][i] == "-S") {
			retn = 1
			break
		}
		if (PROCINFO["argv"][i] == "--") {
			retn = 0
			break
		}		# end of command args (j.i.c)
	}	# loop command line args, NOT ARGV[] user & file args
	return (PROCINFO["sandbox"] = retn)	# set global as well as return
}
#-------------------------------------------
#
function get_ConfigInfo(tmp, LC_ALL, archi)
{
	# called from BEGIN block
	# depricated: rForm["Date:"]=strftime(PROCINFO["strftime"],systime,1); # UTC
	#x rForm["Machine:"]=ENVIRON["PROCESSOR_ARCHITECTURE"]; # eg:"AMD64" "X-86" etc
	rForm["Compiler:"] = "?"
	rForm["Compilation CFLAGS:"] = "?"
	#  OS: linux-gnu
	get_OS()	# Windows, linux-gnu, etc.
	# sets rForm["OS:"]; rForm["platform:"]; rForm["uname output:"];
	archi = ENVIRON["PROCESSOR_ARCHITECTURE"]
	# eg:"AMD64" "X-86" etc
	if (archi ~ /86/) {
		archi = archi "(32-bit gawk)"
	} else {
		if (archi ~ /64/) {
			archi = archi "(64-bit gawk)"
		}
		# AMD64, IA64, ARM64
	}
	# whatever archi is, check the OS
	if (ENVIRON["PROCESSOR_ARCHITEW6432"] ~ /64/) {
		archi = archi " under a 64-bit OS"
	}
	rForm["Machine:"] = archi
	rForm["Gawk Version:"] = gawkVersion()
	# Please see if setting the environment variable LC_ALL to LC_ALL=C causes things to
	# behave as you expect. If so, its a locale issue, and may or may not really be a bug.
	LC_ALL = ENVIRON["LC_ALL"]
	rForm["LC_ALL:"] = (length(LC_ALL) > 0) ? LC_ALL : "<undefined>"
	#------------------------------------------------------------------------
	
	#  ref: "mingw" Microsoft Windows, using 32-bit MinGW
	#  https://sourceforge.net/projects/ezwinports/ # by: eli-zaretskii  2022_0910
	#  https://sourceforge.net/projects/ezwinports/files/gawk-5.2.0-w32-bin.zip/download
	#  https://sourceforge.net/projects/ezwinports/files/gawk-5.2.0-w32-src.zip/download
	#--------------------------
	
	#  in Windows, uname-equivalent is the 'ver' command. >ver Microsoft Windows
	#  uname (short for unix name) is a computer program in Unix and Unix-like computer operating systems that prints the name, version and other details about the current machine and the operating system running on it.
	#  uname output: Linux kurt-OptiPlex-7020 5.15.0-47-generic #51-Ubuntu SMP Thu Aug
	#  11 07:51:15 UTC 2022 x86_64 x86_64 x86_64 GNU/Linux
	#  Machine Type: x86_64-pc-linux-gnu
	# Win 7 SYSINFO System Type:  x64-based PC # me
}
#-------------------------------------------
#
function get_OS(cmd, sysrc, ver_win, wver)
{
	# called from get_ConfigInfo();
	# Check if OS=="Windows_NT" BTW: Same for all Win versions to Win 11 (2022_0922)
	# if(ENVIRON["OS"]!~/Windows_NT/) # some of the config info code may not work ...
	OS_env = ENVIRON["OS"]
	# OS_env expected to be 'Windows_NT' in all cases
	#  PROCINFO["platform"]=    	mingw
	#  Ref: (Gawk UserGuide) 7.5.2 Built-in Variables That Convey Information
	#  PROCINFO["platform"] on Gawk 5.2.0 is one of:
	#  "mingw" Microsoft Windows, using 32-bit MinGW.
	#  "os390" OS/390.
	#  "posix" GNU/Linux, Cygwin, Mac OS X, and legacy Unix systems.
	#  "vms" OpenVMS or Vax/VMS.
	#--------------------------
	#  ref: "mingw" Microsoft Windows, using 32-bit MinGW
	#  https://sourceforge.net/projects/ezwinports/ # by: eli-zaretskii  2022_0910
	#  https://sourceforge.net/projects/ezwinports/files/gawk-5.2.0-w32-bin.zip/download
	#  https://sourceforge.net/projects/ezwinports/files/gawk-5.2.0-w32-src.zip/download
	#--------------------------
	cmd = "ver"
	while ((cmd | getline wver) > 0) {
		if (wver ~ /Version/) {
			break
		}
	}
	if (ERRNO) {
		print("Windows ver command ERRNO=", ERRNO) > "/dev/stderr"
	}
	close(cmd)
	# Example wver == 'Microsoft Windows [Version 10.0.19044.1706]'
	# Example wver == "Microsoft Windows [Version 6.1.7601]";
	# Reference Table for Windows Version Numbers as of 2022_1212
	switch (wver) {	# lookup name of Windows version
	# ref: https://en.wikipedia.org/wiki/List_of_Microsoft_Windows_versions
	case /6.0/:
		ver_win = "Vista"
		break
	case /6.1/:
		ver_win = "7"
		break
	case /6.2/:
		ver_win = "8"
		break
	case /6.3/:
		ver_win = "8.1"
		break
	case /10.0.1/:
		ver_win = "10"
	case /10.0.2/:
		ver_win = "11"
		break
	default:
		ver_win = "Unknown version"
	}
	# end switch()
	ver_win = "Win " ver_win
	# common to all in above switch()
	sub(/Microsoft Windows/, "", wver)	# trim to leave '[Version x.x.xxxx]'
	rForm["OS:"] = sprintf("%s = %s%s", OS_env, ver_win, wver)
	rForm["platform:"] = PROCINFO["platform"]
	rForm["uname output:"] = "?"
}
#-------------------------------------------
#
function get_datetimeUTC(fmt)
{
	# returns timestamp for rForm["Date:"]
	# !!! Different from PROCINFO["strftime"] default "%a %b %e %H:%M:%S %Z %Y"
	fmt = "%a %b %d %Y %H:%M:%S UTC"	# time formatted as UTC
	# eg: Sat Sep 24 2022 05:24:11 UTC
	return strftime(fmt, systime(), "UTC")
}
#-------------------------------------------
function get_user_options(uopt, fmt, outf, cmdwin, chk, i)
{
	# 1) cmd line args set via -v; will not show up in ARGV[]
	rForm["To:"] = (typeof(to) ~ /string/) ? to : "bug-gawk@gnu.org"
	rForm["From:"] = (typeof(from) ~ /string/) ? from : "[YOUR email address here]"
	# 2) test for gawkbug opts [--help] [--version]
	outf = ""	# init null for test later
	for (i = 1; i < ARGC; i++) {
		if (ARGV[i] == "--help") {
			usage()	# never returns
		}
		if (ARGV[i] == "--version") {
# #!/bin/sh - VERSTR="GNU gawkbug, version ${VERSION}"
			printf("GNU gawkbug, version %s\n",rForm["gawkbug ver:"]) > "/dev/stderr"
#			"GNU gawkbug, version win.22.1219"
			exit _abend=1
		}	}
	# 3) test for valid win filename?
	for (i = 1; i < ARGC; i++) {
		outf = chk_eform_filename(ARGV[i])
		if (outf) {
			break	# have a probable Win filename
			# if not, continue looking in ARGV[] ...
		}	
	}
	# 4) test for user supplied filename for gawkbug eform
	if (outf) {
		sysfn["edit_fnam"] = outf	# file to edit and review before emailing.
		# user supplied a filename that may be new or existing. Need to know!
		if (dBug) {
			printf "getuopt if#2 e-fname='%s'\n", sysfn["edit_fnam"]
		}
		cmdwin = sprintf("IF EXIST \"%s\" (ECHO YES) ELSE (ECHO NO)", outf)
		# returns 'YES' or 'NO'; one word, one line
		while ((cmdwin | getline chk) > 0) {
			if (chk ~ /[YN]/) {
				if (dBug) {
					printf "getuopt #3 chk='%s'\n", chk
				}
				break
			}
		}
		close(cmdwin)
		switch (chk) {
		case /Y/:
			pgmFeature["eform_new"] = 0
			break	# do not overwrite
		case /N/:
			pgmFeature["eform_new"] = 1
			break	# create
		default:	# should not happen, system() or OS error??
			printf("gawkbug: Unexpected error with '%s'\n", outf) > "/dev/stderr"
			exit _abend=127
		}
		pgmFeature["validate"] = 1	# all switch cases above
	} else {	# create default filenane for eform;
		# note default path is implied to user's current directory
		# !!! Never "/dev/stdout" or "/dev/stderr" !!! due to main validation loop
		# timestamp for eform and eform file name
		# eg: "C:\foo\Gawkbug\gawkbug.eform.221202.072751.txt"
		fmt = "%y%m%d.%H%M%S"	#yymmdd.HHMMSS == 13 characters, UTC not local
		sysfn["ftimestamp"] = strftime(fmt, systime(), "UTC")
		sysfn["edit_fnam"] = "gawkbug.eform." sysfn["ftimestamp"] ".txt"
		if (dBug) {
			printf "getuopt if#1 e-fname='%s'\n", sysfn["edit_fnam"]
			# assume new
		}
		pgmFeature["eform_new"] = 1
		pgmFeature["validate"] = 1
	}
}
#-------------------------------------------
#
function init4validation()
{
	# (re)init globals for validation after every edit
	# called from BEGIN block and ENDFILE after every re-edit
	#*** setup to read saved report back in, check for valid entries ***
	ARGV[ARGIND + 1] = sysfn["edit_fnam"]
	ARGC++
	split("", valid, FS)	# ensure valid[] is an array;
	# valid is in order of (rptLabels[i],rForm[rptLabels[i]])
	split("", errsarr, FS)	# errsarr[] is an array;
	valid[0] = 0	# init
	split("", nochange, FS)	# nochange[] is an array;
}
#-------------------------------------------
#
function init_rptLabels(strList, i, tmpa)
{
	# called from BEGIN block
	strList = "From:|To:|Subject:|Date:||Configuration Information [Automatically generated, do not change]:||Gawk Version:|Machine:|OS:|platform:|Compiler:|Compilation CFLAGS:|uname output:|LC_ALL:|I have read:|  Yes or No:|gawkbug ver:|Description:|Repeat-By:|Fix:"
	split(strList, rptLabels, "|")	# global arr rptLabels[1]="From:" [2]="To:" ...
	#  for(i in rptLabels){
	#  	rForm[rptLabels[i]]["grp"]=i;	# global arr rForm["From:"]["grp"]=1;"To:"=2
	#  	}
}
#-------------------------------------------
#
function make_boilerplate()
{
	# I have:|Description:|Repeat-By:|Fix:|Subject:
	# separate function to make it easy to update, deal with foreign languages, etc.
	rForm["Subject:"] = "[50 character or so descriptive subject here {for reference}]"
	rForm["I have read:"] = "https://www.gnu.org/software/gawk/manual/html_node/Bugs.html"
	rForm["  Yes or No:"] = "Yes | No  [ Choose one. If 'No', then why haven't you? ]"
	rForm["Description:"] = "[Detailed description of the problem, suggestion, or complaint.]\n"
	rForm["Repeat-By:"] = "[Describe the sequence of events that causes the problem to occur.]\n"
	rForm["Fix:"] = "[Description of how to fix the problem.  If you don't know a fix for the problem, don't include this section.]\n"
}

#------------------------------------------

#>|====================================================================
#>| *** END *** END *** END *** END *** END *** END *** END *** END ***
#>|====================================================================
END {
	if(_abend) exit _abend
# normal end & exit
	tell = sprintf("echo Please email your %s file to %s & pause",\
       sysfn["edit_fnam"], rForm["To:"])
	system(tell)
}
#-------------------------------------------
#
# ============================================================================
#   Copyright (C) 2022 Free Software Foundation, Inc.
#   This Library of AWK source code programs is free software:
#      you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation, either version 3 of the License, or
#      (at your option) any later version.
#   This Library is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#   To get a copy of the GNU General Public License
#      see <https://www.gnu.org/licenses/>.
# ============================================================================

